@@ -3,7 +3,6 @@ import difflib
 import json
 import pathlib
 from functools import cached_property
-from importlib import import_module
 from json import dumps, loads
 from typing import Callable, Dict, List, Optional, Tuple
 from urllib.parse import ParseResult, urlparse
@@ -146,11 +145,7 @@ class BaseSchemaLoader:
     def handle_pk_parameter(  # pylint: disable=no-self-use
         self, resolved_route: ResolverMatch, path: str, method: str
     ) -> Tuple[str, ResolverMatch]:
-        split_view_path = resolved_route.view_name.split(".")
-        view_name = split_view_path.pop()
-        imported_module = import_module(".".join(split_view_path))
-        view = getattr(imported_module, view_name)
-        coerced_path = BaseSchemaGenerator().coerce_path(path=path, method=method, view=view)
+        coerced_path = BaseSchemaGenerator().coerce_path(path=path, method=method, view=resolved_route.func)  # type: ignore
         pk_field_name = "".join(
             list([entry.replace("+ ", "") for entry in difflib.Differ().compare(path, coerced_path) if "+ " in entry])
         )
